/* cmyk4toppm - combine three portable graymaps into one portable pixmap
**
** By Eli the Bearded / Benjamin Elijah Griffin, 2005, based on rgb3toppm.c:
** Copyright (C) 1991 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include "ppm.h"

int
main( argc, argv )
    int argc;
    char* argv[];
    {
    FILE* cfd;
    FILE* yfd;
    FILE* mfd;
    FILE* kfd;
    gray* crow;
    gray* cP;
    gray* yrow;
    gray* yP;
    gray* mrow;
    gray* mP;
    gray* krow;
    gray* kP;
    pixel* pixelrow;
    register pixel* pP;
    int rows, cols, trows, tcols, row, col;
    gray cmaxval, ymaxval, mmaxval, kmaxval;
    int cformat, yformat, mformat, kformat;
    pixval pmaxval;


    ppm_init( &argc, argv );

    if ( argc != 5 )
	pm_usage( "<cyan pgmfile> <magenta pgmfile> <yellow pgmfile> <black pgmfile> " );

    cfd = pm_openr( argv[1] );
    mfd = pm_openr( argv[2] );
    yfd = pm_openr( argv[3] );
    kfd = pm_openr( argv[4] );

    pgm_readpgminit( cfd, &cols, &rows, &cmaxval, &cformat );
    pgm_readpgminit( mfd, &tcols, &trows, &mmaxval, &mformat );
    if ( tcols != cols || trows != rows )
	pm_error( "all four graymaps must be the same size" );
    pgm_readpgminit( yfd, &tcols, &trows, &ymaxval, &yformat );
    if ( tcols != cols || trows != rows )
	pm_error( "all four graymaps must be the same size" );
    pgm_readpgminit( kfd, &tcols, &trows, &kmaxval, &kformat );
    if ( tcols != cols || trows != rows )
	pm_error( "all four graymaps must be the same size" );

    pmaxval = cmaxval;
    if ( mmaxval > pmaxval ) pmaxval = mmaxval;
    if ( ymaxval > pmaxval ) pmaxval = ymaxval;
    if ( kmaxval > pmaxval ) pmaxval = kmaxval;
    crow = pgm_allocrow( cols );
    mrow = pgm_allocrow( cols );
    yrow = pgm_allocrow( cols );
    krow = pgm_allocrow( cols );

    ppm_writeppminit( stdout, cols, rows, pmaxval, 0 );
    pixelrow = ppm_allocrow( cols );

    /* See ppmtocmyk4.c for conversion method comment */
    for ( row = 0; row < rows; row++ )
	{
	pgm_readpgmrow( cfd, crow, cols, cmaxval, cformat );
	pgm_readpgmrow( mfd, mrow, cols, mmaxval, mformat );
	pgm_readpgmrow( yfd, yrow, cols, ymaxval, yformat );
	pgm_readpgmrow( kfd, krow, cols, kmaxval, kformat );

	for ( col = 0, cP = crow, mP = mrow, yP = yrow, kP = krow, pP = pixelrow;
	      col < cols;
	      ++col, ++cP, ++mP, ++yP, ++kP, ++pP )
	    {
	    if ( cmaxval != pmaxval ) *cP = (int) *cP * pmaxval / cmaxval;
	    if ( mmaxval != pmaxval ) *mP = (int) *mP * pmaxval / mmaxval;
	    if ( ymaxval != pmaxval ) *yP = (int) *yP * pmaxval / ymaxval;
	    if ( kmaxval != pmaxval ) *kP = (int) *kP * pmaxval / kmaxval;

	    *cP = MAX(0, (pmaxval - (int) *cP));
	    *mP = MAX(0, (pmaxval - (int) *mP));
	    *yP = MAX(0, (pmaxval - (int) *yP));
	    *cP = MAX(0, ( (int) *cP - (int) *kP ));
	    *mP = MAX(0, ( (int) *mP - (int) *kP ));
	    *yP = MAX(0, ( (int) *yP - (int) *kP ));

	    PPM_ASSIGN( *pP, *cP, *mP, *yP );
	    }
	ppm_writeppmrow( stdout, pixelrow, cols, pmaxval, 0 );
	}

    pm_close( cfd );
    pm_close( mfd );
    pm_close( yfd );
    pm_close( kfd );
    pm_close( stdout );

    exit( 0 );
    }
