/* ppmtocmyk4.c - separate a portable pixmap into four portable graymaps
**
** By Eli the Bearded / Benjamin Elijah Griffin, 2005, based on ppmtorgb3.c:
** Copyright (C) 1991 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#include <string.h>
#include "ppm.h"
#include "pgm.h"

int
main( argc, argv )
    int argc;
    char* argv[];
    {
    FILE* ifp;
    FILE* cynfile;
    FILE* magfile;
    FILE* yelfile;
    FILE* blkfile;
    const char* basename;
    char filename[100];
    char* cp;
    pixel* pixelrow;
    register pixel* pP;
    gray* cynrow;
    gray* magrow;
    gray* yelrow;
    gray* blkrow;
    gray* cP;
    gray* mP;
    gray* yP;
    gray* kP;
    int rows, cols, format, row;
    register int col;
    pixval maxval;
    pixval klevel;
    pixval min_k;


    ppm_init( &argc, argv );

    if ( argc > 2 )
	pm_usage( "[ppmfile]" );

    if ( argc == 2 )
	{
	ifp = pm_openr( argv[1] );
	basename = argv[1];
	cp = strrchr( basename, '.' );
	if ( cp != NULL )
	    *cp = '\0';
	}
    else
	{
	ifp = stdin;
	basename = "noname";
	}

    ppm_readppminit( ifp, &cols, &rows, &maxval, &format );
    pixelrow = ppm_allocrow( cols );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".cyn" );
    cynfile = pm_openw( filename );
    pgm_writepgminit( cynfile, cols, rows, (gray) maxval, 0 );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".mag" );
    magfile = pm_openw( filename );
    pgm_writepgminit( magfile, cols, rows, (gray) maxval, 0 );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".yel" );
    yelfile = pm_openw( filename );
    pgm_writepgminit( yelfile, cols, rows, (gray) maxval, 0 );
    (void) strcpy( filename, basename );
    (void) strcat( filename, ".blk" );
    blkfile = pm_openw( filename );
    pgm_writepgminit( blkfile, cols, rows, (gray) maxval, 0 );

    cynrow = pgm_allocrow( cols );
    magrow = pgm_allocrow( cols );
    yelrow = pgm_allocrow( cols );
    blkrow = pgm_allocrow( cols );

    /* RGB -> CMY -> CMYK:
     *
     * c      = maxval - r
     * m      = maxval - g
     * y      = maxval - b
     *
     * klevel = min(c,m,y)
     *
     * min_k  = maxval / 2
     * if ( klevel >= min_k ) { k = klevel } else { k = 0 }
     *
     * c      = c - k
     * m      = m - k
     * y      = y - k
     *
     * Reverse to go from CMY or CMYK to RGB.
     * Choice of min_k is typically ink dependent.
     */

    min_k = maxval / 2;
    for ( row = 0; row < rows; ++row )
	{
	ppm_readppmrow( ifp, pixelrow, cols, maxval, format );

	for ( col = 0, pP = pixelrow,
	      cP = cynrow, mP = magrow, yP = yelrow, kP = blkrow;
	      col < cols;
	      ++col, ++pP, ++cP, ++mP, ++yP, ++kP )
	{
	    *cP = (gray) maxval - (gray) PPM_GETR( *pP );
	    *mP = (gray) maxval - (gray) PPM_GETG( *pP );
	    *yP = (gray) maxval - (gray) PPM_GETB( *pP );

	    klevel = MIN(MIN(*cP, *mP), *yP);

	    if (klevel >= min_k)
	    { 
	      *cP -= klevel;
	      *mP -= klevel;
	      *yP -= klevel;
	      *kP  = klevel;
	    }
	    else
	    {
	      *kP = 0;
	    }
	}


	pgm_writepgmrow( cynfile, cynrow, cols, maxval, 0 );
	pgm_writepgmrow( magfile, magrow, cols, maxval, 0 );
	pgm_writepgmrow( yelfile, yelrow, cols, maxval, 0 );
	pgm_writepgmrow( blkfile, blkrow, cols, maxval, 0 );

	}

    pm_close( ifp );
    pm_close( cynfile );
    pm_close( yelfile );
    pm_close( magfile );

    exit( 0 );
    }
